@@ -17,7 +17,7 @@ import time
 import datetime
 from .data_process import trainset, test_preprocess_chooseOne, testset, multibatch_test_save, singlebatch_test_save
 from skimage import io
-from deepcad.movie_display import img_display
+from deepcad.movie_display import test_img_display
 
 
 class training_class():
@@ -48,9 +48,9 @@ class training_class():
         self.patch_t = 150
         self.patch_x = 150
         self.patch_y = 150
-        self.gap_y = 75
-        self.gap_x = 75
-        self.gap_t = 75
+        self.gap_y = 115
+        self.gap_x = 115
+        self.gap_t = 115
         self.lr = 0.00001
         self.b1 = 0.5
         self.b2 = 0.999
@@ -60,7 +60,7 @@ class training_class():
         self.scale_factor = 1
         self.train_datasets_size = 2000
         self.select_img_num = 1000
-        self.test_datasize = 500
+        self.test_datasize = 400  # how many slices to be tested (use the first image in the folder by default)
         self.visualize_images_per_epoch = False
         self.save_test_images_per_epoch = False
         self.set_params(params_dict)
@@ -70,7 +70,6 @@ class training_class():
         General function for training DeepCAD network.
 
         """
-
         # create some essential file for result storage
         self.prepare_file()
         # crop input tiff file into 3D patches
@@ -103,13 +102,18 @@ class training_class():
 
     def set_params(self, params_dict):
         """
-        Set the params set by user to the training class object
+        Set the params set by user to the training class object and calculate some default parameters for training
 
         """
         for key, value in params_dict.items():
             if hasattr(self, key):
                 setattr(self, key, value)
 
+        self.gap_x = int(self.patch_x * (1 - self.overlap_factor))  # patch gap in x
+        self.gap_y = int(self.patch_y * (1 - self.overlap_factor))  # patch gap in y
+        self.gap_t = int(self.patch_t * (1 - self.overlap_factor))  # patch gap in t
+        self.ngpu = str(self.GPU).count(',') + 1                    # check the number of GPU used for training
+        self.batch_size = self.ngpu                                 # By default, the batch size is equal to the number of GPU for minimal memory consumption
         print('\033[1;31mTraining parameters -----> \033[0m')
         print(self.__dict__)
 
@@ -171,13 +175,12 @@ class training_class():
             self.whole_x = noise_im.shape[2]
             self.whole_y = noise_im.shape[1]
             self.whole_t = noise_im.shape[0]
-            print('Noise image size -----> ')
-            print('whole_x -----> ', self.whole_x,', whole_y -----> ', self.whole_y,', whole_t -----> ', self.whole_t)
-            # calculate real gap_t
+            print('Noise images shape -----> ', noise_im.shape)
+            # Calculate real gap_t
             self.get_gap_t()
-            # no preprocessing
+            # No preprocessing
             noise_im = noise_im.astype(np.float32) / self.scale_factor
-            # minus min before training
+            # Minus min before training
             # noise_im = (noise_im-noise_im.min()).astype(np.float32)/args.scale_factor
             self.noise_im_all.append(noise_im)
             patch_t2 = self.patch_t * 2
@@ -203,7 +206,6 @@ class training_class():
                         self.coordinate_list[patch_name] = single_coordinate
                         self.stack_index.append(ind)
             ind = ind + 1
-            # return  name_list, noise_im_all, coordinate_list, stack_index
 
     def save_yaml_train(self):
         """
@@ -269,7 +271,7 @@ class training_class():
             train_data = trainset(self.name_list, self.coordinate_list, self.noise_im_all, self.stack_index)
             trainloader = DataLoader(train_data, batch_size=self.batch_size, shuffle=True, num_workers=self.num_workers)
             for iteration, (input, target) in enumerate(trainloader):
-                # the input volume and corresponding target volume from data loader to train the deep neural network
+                # The input volume and corresponding target volume from data loader to train the deep neural network
                 input = input.cuda()
                 target = target.cuda()
                 real_A = input
@@ -279,11 +281,12 @@ class training_class():
                 L1_loss = L1_pixelwise(fake_B, real_B)
                 L2_loss = L2_pixelwise(fake_B, real_B)
                 optimizer_G.zero_grad()
+
                 # Calculate total loss
                 Total_loss = 0.5 * L1_loss + 0.5 * L2_loss
                 Total_loss.backward()
                 optimizer_G.step()
-                # record and estimate the remaining time
+                # Record and estimate the remaining time
                 batches_done = epoch * len(trainloader) + iteration
                 batches_left = self.n_epochs * len(trainloader) - batches_done
                 time_left = datetime.timedelta(seconds=int(batches_left * (time.time() - prev_time)))
@@ -309,10 +312,11 @@ class training_class():
                     print('\n', end=' ')
 
                 if (iteration + 1) % (len(trainloader)) == 0:
-                    # save model at the end of every epoch
+                    # Save model at the end of every epoch
                     self.save_model(epoch, iteration)
-                    # start inference using the denoise model at the end of every epoch (optional)
+                    # Start inference using the denoise model at the end of every epoch (optional)
                     if (self.visualize_images_per_epoch | self.save_test_images_per_epoch):
+                        print('\033[1;31mTesting epoch {} model on the first noisy file -----> \033[0m'.format(epoch))
                         self.test(epoch, iteration)
         print('Train finished. Save all model to disk.')
 
@@ -407,10 +411,11 @@ class training_class():
         # Normalize and display inference image
         if (self.visualize_images_per_epoch):
             display_length = self.test_datasize
-            img_display(output_img, display_length=display_length, norm_min_percent=1, norm_max_percent=98)
+            test_img_display(output_img, display_length=display_length, norm_min_percent=1, norm_max_percent=98)
 
         # Save inference image
         if (self.save_test_images_per_epoch):
+            output_img = output_img[50:self.test_datasize-50, :, :]
             output_img = output_img.astype('int16')
             result_name = self.pth_path + '//' + test_im_name.replace('.tif', '') + '_' + 'E_' + str(
                 train_epoch + 1).zfill(2) + '_Iter_' + str(train_iteration + 1).zfill(4) + '.tif'
